<?php

/* access.php - SynApp2 authentication and database access control
**
   +----------------------------------------------------------------------+
   | SynApp2 Version 1                                                    |
   +----------------------------------------------------------------------+
   | Copyright (c) 2007 - 2009 Richard Howell. All rights reserved.       |
   +----------------------------------------------------------------------+
   | This source file is subject to version 1.01 of the SynApp2 license,  |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
   | http://www.synapp2.org/license/1_01.txt                              |
   | If you did not receive a copy of the SynApp2 license and are unable  |
   | to obtain it through the world-wide-web, please send a note to       |
   | license@synapp2.org so we can mail you a copy immediately.           |
   +----------------------------------------------------------------------+
   | Authors: Richard Howell                                              |
   |                                                                      |
   +----------------------------------------------------------------------+
**
** http://www.synapp2.org
**
** $Id: access.php,v 1.5 2009/01/05 02:39:54 richard Exp $
*/

// SECURITY: <-- search this file for all instances of this tag and make changes as directed
// DEFAULT: <-- search this file for all instances of this tag and make changes as needed

class access_base
{
    //// begin private data members

    // private:

    var $m_action = null;
    var $m_dbx = null;
    var $m_custom = null;
    var $m_link = false;
    var $m_auth = null;

    //// begin public interface

    // public:

    function is_authenticated()
    {
        $token = $this->get_token();

        return !empty($token);
    }

    function select_db($db_name)
    {
        $select_result = false;

        if ($this->db_connect())
        {
            $select_result = $this->m_dbx->select_db($db_name);
        }

        $this->select_db_diagnostic($db_name, $select_result);

        return $select_result;
    }

    //// begin protected interface

    // protected:

    function access_base(&$action, &$dbx, &$custom)
    {
        $this->m_action = &$action;
        $this->m_dbx = &$dbx;
        $this->m_custom = &$custom;
    }

    //// begin private interface

    // private:

    function select_db_diagnostic($db_name, $select_result)
    {
        $enable = true; // false; // DEFAULT: change as appropriate for your needs // SECURITY: consider the value upon deployment

        if ($enable && $this->m_action->is_show_response()) // comment out this line, during debugging, to force message display
        {
            add_debug_msg("select_db('$db_name') - attempt " . ($select_result === false ? 'failed' : 'SUCCEEDED'));
        }
    }

    function get_site_prefix($host = null)
    {
        if (!$host)
        {
            $host = strtolower($_SERVER['HTTP_HOST']); // DEFAULT: change as appropriate for your needs
        }

        $map_prefix = array();

        //// DEFAULT: change as appropriate for your needs
        //
        // $map_prefix['www.yoursite.com'] = 'prefix';
        $map_prefix['synapp2.org'] = 's2admin_';
        $map_prefix['www.synapp2.org'] = 's2admin_';

        return isset($map_prefix[$host]) ? $map_prefix[$host] : '' ;
    }

    function get_database_logical($alternate_appid = null)
    {
        return $this->m_custom->get_database($alternate_appid);
    }

    function get_database_physical($alternate_appid = null, $host = null)
    {
        if (!$host)
        {
            $host = strtolower($_SERVER['HTTP_HOST']); // DEFAULT: change as appropriate for your needs
        }

        $map_physical = array();

        //// DEFAULT: change as appropriate for your needs
        //
        // $map_physical['www.yoursite.com']['logical'] = 'physical';
        // $map_physical['www.yoursite.com']['yourappdata'] = 'accountname_yourappdata';

        ////

        $logical = $this->get_database_logical($alternate_appid);
        $physical = $logical;

        if (!empty($logical))
        {
            $physical = isset($map_physical[$host][$logical]) ? $map_physical[$host][$logical] : ($this->get_site_prefix() . $logical) ;
        }

        return $physical;
    }

    function get_username_physical($logical)
    {
        return $this->get_site_prefix() . $logical;
    }

    function connect_diagnostic(&$link, $hostname, $username, $auth_connect = false)
    {
        $enable = true; // false; // DEFAULT: change as appropriate for your needs // SECURITY: consider the value upon deployment

        if ($enable && $this->m_action->is_show_response()) // comment out this line, during debugging, to force message display
        {
            add_debug_msg(($auth_connect ? 'auth_connect' : 'db_connect') . "('$hostname', '$username') - attempt " . ($link === false ? 'failed' : 'SUCCEEDED'));
        }
    }

    function auth_connect()
    {
        $_one_for_all_hostname = 'localhost'; // DEFAULT: change as appropriate for your needs
        $_one_for_all_username = $this->m_auth->database();
        $_one_for_all_password = 'password'; // SECURITY: change value upon deployment

        //// DEFAULT: change as appropriate for your needs
        //
        $map_credentials['_one_for_all_'] = array($_one_for_all_hostname, $_one_for_all_username, $_one_for_all_password);
//        $map_credentials['www.yoursite.com'] = array($_one_for_all_hostname, 'auth_username', $_one_for_all_password);

        ////

        $host = strtolower($_SERVER["HTTP_HOST"]); // DEFAULT: change as appropriate for your needs

        $credentials = !empty($map_credentials[$host]) ? $map_credentials[$host] : $map_credentials['_one_for_all_'] ;

        $link = $this->m_dbx->connect($credentials[1], $credentials[2], $credentials[0]);

        $this->connect_diagnostic($link, $credentials[0], $credentials[1], true);

        return $link;
    }

    function validation_diagnostic($username, $user_validated)
    {
        $enable = true; // false; // DEFAULT: change as appropriate for your needs // SECURITY: consider the value upon deployment

        if ($enable && $this->m_action->is_show_response()) // comment out this line, during debugging, to force message display
        {
            $msg = "user validation " . ($user_validated ? 'SUCCEEDED' : 'FAILED') .
                   " against " . $this->m_auth->database() . '.' . $this->m_auth->table() .
                   " for " . $this->m_auth->field_username() . " = '$username'" .
                   " and " . $this->m_auth->field_password() . " = '*******'";

            add_debug_msg($msg);
        }
    }

    function authenticate()
    {
        $u = $this->m_action->get_login_username();
        $p = $this->m_action->get_login_password();

        if (!empty($u) || !empty($p)) // detect if session login is occuring
        {
            $this->set_token(); // cancel any previous session login

            $link = $this->auth_connect();

            if ($link)
            {
                if ($this->m_auth)
                {
                    $u = $this->m_dbx->escape_sql_term($u);
                    $p = $this->m_dbx->escape_sql_term($p);

                    $this->m_dbx->select_db($this->m_auth->database());

                    $q = "select count(*) from " . $this->m_auth->table() .
                         " where " . $this->m_auth->field_username() . " = '$u'" .
                         " and " . $this->m_auth->field_password() . " = " . $this->m_auth->function_encrypt($p);

                    $v = $this->m_dbx->get_query_value($q);

                    $user_validated = 1 == $this->m_dbx->get_query_value($q);

                    $this->validation_diagnostic($u, $user_validated);

                    if ($user_validated)
                    {
                        $default_hash = '1a2b3c'; // SECURITY: $default_hash - change all literal instance values upon deployment
                        $hash = $this->resolve($u, $default_hash);
                        $this->set_token($hash); // session login successful
                        $this->set_username($u);
                    }
                }

                $this->m_dbx->close();
            }
        }

        return $this->is_authenticated();
    }

    function set_token($token = null)
    {
        if (empty($token))
        {
            unset_session_var('token');
            $this->set_username();
        }
        else
        {
            set_session_var('token', $token);
        }
    }

    function get_token()
    {
        return get_session_var('token');
    }

    function set_username($username = null)
    {
        if (empty($username))
        {
            unset_session_var('username');
        }
        else
        {
            set_session_var('username', $username);
        }
    }

    function resolve($map_key, $default_hash = null)
    {
        $_one_for_all_hostname = 'localhost'; // DEFAULT: change as appropriate for your needs
        $_one_for_all_username = !$this->get_site_prefix() || !$this->get_database_physical() ? $this->get_username_physical(SYNAPP2) : $this->get_database_physical() ;
        $_one_for_all_password = 'password'; // SECURITY: change value upon deployment

        //// DEFAULT: change as appropriate for your needs
        //
        $map_credentials['1a2b3c'] = array(null, $_one_for_all_hostname, $_one_for_all_username, $_one_for_all_password); // SECURITY: $default_hash - change all literal instance values upon deployment
//        $map_credentials['a1b2c3'] = array(null, $_one_for_all_hostname, 'read_only_user', $_one_for_all_password);
//        $map['special_hash'] = array('special_user', 'host', -u, -p); // customization

        ////

        $result = null;

        if (!empty($map_key))
        {
            if (!$default_hash)
            {
                // map hash to connection credential
                //
                if (isset($map_credentials[$map_key]))
                {
                    $result = $map_credentials[$map_key];
                }
            }
            else
            {
                // map username to hash (of appropriate connection credential)
                //
                foreach ($map_credentials as $hash => $credentials)
                {
                    if ($credentials[0] === $map_key)
                    {
                        $result = $hash;
                        break;
                    }
                }

                if (!$result && isset($map_credentials[$default_hash]))
                {
                    $result = $default_hash;
                }
            }
        }

        return $result;
    }

    function db_connect($force = false)
    {
        if ($force)
        {
            $this->m_link = false;
        }

        if ($this->m_link === false)
        {
            $credential = $this->resolve($this->get_token());

            if (!empty($credential))
            {
                $this->m_link = $this->m_dbx->connect($credential[2], $credential[3], $credential[1]);
                $this->connect_diagnostic($this->m_link, $credential[1], $credential[2]);
            }
        }

        return $this->m_link !== false;
    }
};

class access_app extends access_base
{
    function auth_connect()
    {
        $logical = $this->get_database_logical();
        $physical = $this->get_database_physical();

        //// DEFAULT: change as appropriate for your needs
        //
        $map_auth['_one_for_all_'] = new auth_interface_def($physical);
        // $map_auth['logical'] = new auth_interface_def($physical, 'user_table', 'user_fieldname', 'password_fieldname', 'encrypt_function_name');

        ////

        $this->m_auth = isset($map_auth[$logical]) ? $map_auth[$logical] : $map_auth['_one_for_all_'] ;

        return parent::auth_connect();
    }
};

class access_synapp2 extends access_base
{
    function auth_connect()
    {
        $alternate_appid = SYNAPP2; // DEFAULT: change as appropriate for your needs

        $physical = $this->get_database_physical($alternate_appid);

        $this->m_auth = new auth_interface_def($physical, 'users', 'username');

        ////

        return parent::auth_connect();
    }
};

class access_enterprise extends access_base
{
    function authenticate()
    {
        $u = $this->m_action->get_login_username();
        $p = $this->m_action->get_login_password();

        if (!empty($u) || !empty($p))
        {
            $this->set_token();

            $authenticated = false; // enterprise_authentication($u, $p);

            if ($authenticated)
            {
                $default_hash = '1a2b3c'; // SECURITY: $default_hash - change all literal instance values upon deployment
                $hash = $this->resolve($u, $default_hash);
                $this->set_token($hash);
                $this->set_username($u);
            }
        }

        return $this->is_authenticated();
    }
};

class access_direct extends access_base
{
    function authenticate()
    {
        $u = $this->m_action->get_login_username();
        $p = $this->m_action->get_login_password();

        if (!empty($u) || !empty($p))
        {
            $this->set_token();

            $authenticated = true; // bypass authentication layer

            if ($authenticated)
            {
                $_one_for_all_hostname = 'localhost'; // DEFAULT: change as appropriate for your needs
                $default_hash = 'direct'; // SECURITY: $default_hash - change all literal instance values upon deployment
                $hash = array(null, $_one_for_all_hostname, $u, $this->obfuscator($p));
                $this->set_token($hash);
                $this->set_username($u);
            }
        }

        return $this->is_authenticated();
    }

    function db_connect($force = false)
    {
        if ($force)
        {
            $this->m_link = false;
        }

        if ($this->m_link === false)
        {
            $credential = $this->get_token();

            if (!empty($credential))
            {
                $this->m_link = $this->m_dbx->connect($credential[2], $this->obfuscator($credential[3], true), $credential[1]);

                $this->connect_diagnostic($this->m_link,  $credential[1], $credential[2]);

                if (!$this->m_link)
                {
                    $this->set_token();
                }
            }
        }

        return $this->m_link !== false;
    }

    function obfuscator($subject, $decode = false)
    {
        return $subject; // SECURITY: implement encode/decode
    }
};

////

class auth_interface_def
{
    // customize via instances of this class, not by changing the member initialization
    //
    var $m_database = '';
    var $m_table = 'user';
    var $m_field_username = 'User';
    var $m_field_password = 'Password';
    var $m_function_encrypt = 'password'; // 'oldpassword'; '';

    function auth_interface_def($database,
                                $table = null,
                                $field_username = null,
                                $field_password = null,
                                $function_encrypt = null)
    {
        $this->database($database);
        $this->table($table);
        $this->field_username($field_username);
        $this->field_password($field_password);
        $this->function_encrypt('', $function_encrypt);
    }

    function database($database = null)
    {
        if (!empty($database))
        {
            $this->m_database = $database;
        }

        return $this->m_database;
    }

    function table($table = null)
    {
        if (!empty($table))
        {
            $this->m_table = $table;
        }

        return $this->m_table;
    }

    function field_username($field_username = null)
    {
        if (!empty($field_username))
        {
            $this->m_field_username = $field_username;
        }

        return $this->m_field_username;
    }

    function field_password($field_password = null)
    {
        if (!empty($field_password))
        {
            $this->m_field_password = $field_password;
        }

        return $this->m_field_password;
    }

    function function_encrypt($value, $function_encrypt = null)
    {
        if (!empty($function_encrypt))
        {
            $this->m_function_encrypt = $function_encrypt;
        }

        return $this->m_function_encrypt . "('$value')";
    }
};

////

function &new_access(&$action, &$dbx, &$custom)
{
    $obj = null;

    $login_method = $action->get_login_method() ? $action->get_login_method() : (dbx_base::get_engine() == ENGINE_OCI ? 'direct' : '') ; // TODO: TBD?? change where we do this?

    switch ($login_method)
    {
    case 'app':
        $obj = new access_app($action, $dbx, $custom);
        break;
    case SYNAPP2:
    default:
        $obj = new access_synapp2($action, $dbx, $custom);
        break;
    case 'direct':
        $obj = new access_direct($action, $dbx, $custom);
        break;
    case 'enterprise':
        $obj = new access_enterprise($action, $dbx, $custom);
        break;
    }

    $obj->authenticate();

    return $obj;
}

////

?>
