<?php

/* custom.php - SynApp2 manager for application specific customization
**
   +----------------------------------------------------------------------+
   | SynApp2 Version 1                                                    |
   +----------------------------------------------------------------------+
   | Copyright (c) 2007 - 2009 Richard Howell. All rights reserved.       |
   +----------------------------------------------------------------------+
   | This source file is subject to version 1.01 of the SynApp2 license,  |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
   | http://www.synapp2.org/license/1_01.txt                              |
   | If you did not receive a copy of the SynApp2 license and are unable  |
   | to obtain it through the world-wide-web, please send a note to       |
   | license@synapp2.org so we can mail you a copy immediately.           |
   +----------------------------------------------------------------------+
   | Authors: Richard Howell                                              |
   |                                                                      |
   +----------------------------------------------------------------------+
**
** http://www.synapp2.org
**
** $Id: custom.php,v 1.7 2009/02/21 00:38:22 richard Exp $
*/

class custom
{
    var $m_dbx = null;
    var $m_appid = null;
    var $m_qid = null;
    var $m_data = null;

    function custom(&$dbx, $appid, $qid)
    {
        $this->m_dbx = &$dbx;
        $this->m_appid = $appid;
        $this->m_qid = $qid;
        $this->build();
    }

    function build()
    {
        define('SYNAPP2', 'synapp2');
        define('APPID', '_appid_');
        define('QID', '_qid_');

        define('DATABASE', '_database_');
        define('TABLE', '_table_');
        define('ORDER', '_order_');
        define('MACRO', '_macro_');
        define('LIST_MACRO', '_list_macro_');
        define('LIST_MACRO_ORDER', '_list_macro_order_');
        define('LIST_MACRO_FILTER', '_list_macro_filter_');
        define('EXTRA', '_extra_');
        define('OMIT', '_omit_');
        define('INCL', '_incl_');
        define('QUERY', '_query_');
        define('SUBQUERY', '_subquery_');
        define('FETCH', '_fetch_');
        define('LEGEND', '_legend_');
        define('DETAIL_SUMMARY_COLS', '_detail_summary_cols_');

        define('CUSTOM', '_custom_');
        define('KEYMAP', '_keymap_');
        define('HTMLDBPT', 'HTMLDB_PLAN_TABLE');

        define('TABLE_NAME', 'table_name');
        define('COL_NAME', 'col_name');
        define('JOIN_TABLE', 'join_table');
        define('JOIN_COL', 'join_col');

        define('CONSTRAINT', 'constraint');
        define('OFFSET', 'offset');

        define('PATH', 'path');
        define('BASIS_INDEX', 'basis_index');

        define('DEFAULT_PK_NAME', 'id');
        define('DEFAULT_FK_PREFIX', 'id_');

        define('SHARED', '_shared_');
        define('COMMENT', '_RESERVED_'); // reserved to support field comment dependencies a'la MySQL, for other DB engines

        define('IFORM', 'iform');
        define('DFORM', 'dform');
        define('SFORM', 'sform');
        define('TFORM', 'tform');
        define('AFORM', 'aform');
        define('FFORM', 'fform');
        define('RFORM', 'rform');

        define('NAV', '_nav_');
        define('RPT', '_rpt_');

        ////

        $custom_inc = 'custom.inc.php';

        include($custom_inc);

        $app_custom_inc = "../{$this->m_appid}/{$custom_inc}";

        if (file_exists($app_custom_inc))
        {
            include($app_custom_inc);
        }

        ////

        $this->m_data[APPID]['login'][DATABASE] = '';
    }

    function put_pk_keymaps($pkeys)
    {
        if (!empty($pkeys))
        {
            foreach ($pkeys as $table_name => $col_name)
            {
                $pk_name = $this->get_pk_name($table_name);

                if ($pk_name == DEFAULT_PK_NAME && $pk_name != $col_name)
                {
                    $this->set_keymap($table_name, $col_name, $table_name, $col_name);
                }
            }
        }
    }

    function get_pk_name($table_name, $qualified=false, $alternate_appid = null)
    {
        $result = null;

        $database = $this->get_database($alternate_appid);

        if ($keymaps = $this->get_keymaps($database))
        {
            foreach ($keymaps as $keymap)
            {
                if (2 == count(array_intersect_assoc(array(TABLE_NAME=>$table_name, JOIN_TABLE=>$table_name), $keymap)))
                {
                    $result = $keymap[COL_NAME];
                    break;
                }
            }
        }

        if (empty($result))
        {
            $result = $this->m_dbx->get_query_value($this->m_dbx->get_keymap_table_query(
                                                    "select " . COL_NAME . " as \"" . COL_NAME . "\" from " . get_database_physical($alternate_appid) . '.' . KEYMAP .
                                                    " where " . TABLE_NAME . "='$table_name' and " . JOIN_TABLE . "='$table_name'"));
        }

        if (empty($result) && !empty($table_name))
        {
            $result = DEFAULT_PK_NAME;
        }

        return $qualified ? "$table_name.$result" : $result ;
    }

    function get_parent_name($table_name, $fk_name, $alternate_appid = null)
    {
        $result = null;

        $database = $this->get_database($alternate_appid);

        if ($keymaps = $this->get_keymaps($database))
        {
            foreach ($keymaps as $keymap)
            {
                if (2 == count(array_intersect_assoc(array(TABLE_NAME=>$table_name, COL_NAME=>$fk_name), $keymap)))
                {
                    if ($keymap[JOIN_TABLE] != $table_name)
                    {
                        $result = $keymap[JOIN_TABLE];
                        break;
                    }
                }
            }
        }

        if (empty($result))
        {
            $result = $this->m_dbx->get_query_value($this->m_dbx->get_keymap_table_query(
                                                    "select " . JOIN_TABLE . " as \"" . JOIN_TABLE . "\" from " . get_database_physical($alternate_appid) . '.' . KEYMAP .
                                                    " where " . TABLE_NAME . "='$table_name' and " . COL_NAME . "='$fk_name' and " . JOIN_TABLE . "!='$table_name'"));
        }

        if (empty($result) && strpos($fk_name, DEFAULT_FK_PREFIX) === 0)
        {
            $result = substr($fk_name, strlen(DEFAULT_FK_PREFIX));
        }

        return $result;
    }

    function get_parent_join_col($parent_name, $child_name, $child_col, $alternate_appid = null)
    {
        $result = null;

        $database = $this->get_database($alternate_appid);

        if ($keymaps = $this->get_keymaps($database))
        {
            foreach ($keymaps as $keymap)
            {
                if (3 == count(array_intersect_assoc(array(TABLE_NAME=>$child_name, COL_NAME=>$child_col, JOIN_TABLE=>$parent_name), $keymap)))
                {
                    $result = $keymap[JOIN_COL];
                    break;
                }
            }
        }

        if (empty($result))
        {
            $result = $this->m_dbx->get_query_value($this->m_dbx->get_keymap_table_query(
                                                    "select " . JOIN_COL . " as \"" . JOIN_COL . "\" from " . get_database_physical($alternate_appid) . '.' . KEYMAP .
                                                    " where " . TABLE_NAME . "='$child_name' and " . COL_NAME . "='$child_col' and " . JOIN_TABLE . "='$parent_name'"));
        }

        if (empty($result) && !empty($parent_name) && !empty($child_name) && !empty($child_col))
        {
            $result = DEFAULT_PK_NAME;
        }

        return $result;
    }

    function get_child_join_cols($parent_name, $child_name, $alternate_appid = null)
    {
        $result = null;

        $database = $this->get_database($alternate_appid);

        if ($keymaps = $this->get_keymaps($database))
        {
            foreach ($keymaps as $keymap)
            {
                if (2 == count(array_intersect_assoc(array(TABLE_NAME=>$child_name, JOIN_TABLE=>$parent_name), $keymap)))
                {
                    $result[] = $keymap[COL_NAME];
                }
            }
        }

        if (empty($result))
        {
            $recs = $this->m_dbx->get_query_records($this->m_dbx->get_keymap_table_query(
                                                    "select " . COL_NAME . " as \"" . COL_NAME . "\" from " . get_database_physical($alternate_appid) . '.' . KEYMAP .
                                                    " where " . TABLE_NAME . "='$child_name' and " . JOIN_TABLE . "='$parent_name'"));
            if (!empty($recs))
            {
                foreach ($recs as $rec)
                {
                    $result[] = $rec[COL_NAME];
                }
            }
        }

        if (empty($result) && !empty($parent_name) && !empty($child_name))
        {
            $result[] = DEFAULT_FK_PREFIX . $parent_name;
        }

        return $result;
    }

    function get_key_cols($table_name, $alternate_appid = null)
    {
        $result = null;

        $database = $this->get_database($alternate_appid);

        if ($keymaps = $this->get_keymaps($database))
        {
            foreach ($keymaps as $keymap)
            {
                if (1 == count(array_intersect_assoc(array(JOIN_TABLE=>$table_name), $keymap)))
                {
                    $result[] = $keymap[JOIN_COL];
                }
            }
        }

        if (empty($result))
        {
            $recs = $this->m_dbx->get_query_records($this->m_dbx->get_keymap_table_query(
                                                    "select " . JOIN_COL . " as \"" . JOIN_COL . "\" from " . get_database_physical($alternate_appid) . '.' . KEYMAP .
                                                    " where " . JOIN_TABLE . "='$table_name'"));
            if (!empty($recs))
            {
                foreach ($recs as $rec)
                {
                    $result[] = $rec[JOIN_COL];
                }
            }
        }

        if (empty($result) && !empty($table_name))
        {
            $result[] = DEFAULT_PK_NAME;
        }

        return $result;
    }

    function get_sql_calc_found_rows()
    {
        return $this->m_dbx->get_sql_calc_found_rows();
    }

    function get_appid($alternate_appid = null)
    {
        return !empty($alternate_appid) ? $alternate_appid : $this->m_appid ;
    }

    function get_database($alternate_appid = null)
    {
        $appid = $this->get_appid($alternate_appid);
        return isset($this->m_data[APPID][$appid][DATABASE]) ? $this->m_data[APPID][$appid][DATABASE] : $appid ;
    }

    function get_keymaps($database)
    {
        return !empty($this->m_data[DATABASE][$database][KEYMAP]) ? $this->m_data[DATABASE][$database][KEYMAP] : null ;
    }

    function set_keymap($table_name, $col_name, $join_table, $join_col, $alternate_appid = null)
    {
        if ($database = $this->get_database($alternate_appid))
        {
            $entry = array(TABLE_NAME=>$table_name, COL_NAME=>$col_name, JOIN_TABLE=>$join_table, JOIN_COL=>$join_col);

            if ($keymaps = $this->get_keymaps($database))
            {
                foreach ($keymaps as $i => $keymap)
                {
                    if (2 == count(array_intersect_assoc(array(TABLE_NAME=>$table_name, COL_NAME=>$col_name), $keymap)))
                    {
                        $this->m_data[DATABASE][$database][KEYMAP][$i] = $entry;
                        $entry = null;
                        break;
                    }
                }
            }

            if ($entry)
            {
                $this->m_data[DATABASE][$database][KEYMAP][] = $entry;
            }
        }
    }

    function get_qid($alternate_qid)
    {
        return !empty($alternate_qid) ? $alternate_qid : $this->m_qid ;
    }

    function get_table($alternate_qid = null)
    {
        $appid = $this->get_appid();
        $qid = $this->get_qid($alternate_qid);
        return !empty($this->m_data[APPID][$appid][QID][$qid][TABLE]) ? $this->m_data[APPID][$appid][QID][$qid][TABLE] : $qid ;
    }

    function get_query($alternate_qid = null)
    {
        $appid = $this->get_appid();
        $qid = $this->get_qid($alternate_qid);
        return !empty($this->m_data[APPID][$appid][QID][$qid][QUERY]) ? $this->m_data[APPID][$appid][QID][$qid][QUERY] : null ;
    }

    function get_subquery($alternate_qid = null)
    {
        $appid = $this->get_appid();
        $qid = $this->get_qid($alternate_qid);
        return !empty($this->m_data[APPID][$appid][QID][$qid][SUBQUERY]) ? $this->m_data[APPID][$appid][QID][$qid][SUBQUERY] : null ;
    }

    function get_fetch($col, $alternate_qid = null)
    {
        $appid = $this->get_appid();
        $qid = $this->get_qid($alternate_qid);
        return !empty($this->m_data[APPID][$appid][QID][$qid][FETCH][$col]) ? $this->m_data[APPID][$appid][QID][$qid][FETCH][$col] : null ;
    }

    function get_fetch_cols($alternate_qid = null)
    {
        $appid = $this->get_appid();
        $qid = $this->get_qid($alternate_qid);
        return !empty($this->m_data[APPID][$appid][QID][$qid][FETCH]) ? $this->m_data[APPID][$appid][QID][$qid][FETCH] : null ;
    }

    function get_order($alternate_qid = null, $alternate_order = null)
    {
        $appid = $this->get_appid();
        $qid = $this->get_qid($alternate_qid);
        $table_name = $this->get_table($alternate_qid);

        ////

        if (!empty($alternate_order) && is_array($tokens = explode(',', $alternate_order))) // col1,col2,za,col3
        {
            $alternate_order_expr = '';
            $sep = '';
            $za = false;

            $other_cols = array();

            if (!empty($this->m_data[APPID][$appid][QID][$qid][EXTRA]))
            {
                $recs = $this->m_data[APPID][$appid][QID][$qid][EXTRA];

                if (!empty($recs))
                {
                    foreach ($recs as $basis_col => $basis_recs)
                    {
                        foreach ($basis_recs as $col_name => $col_expr)
                        {
                            $other_cols[$col_name] = $col_name;
                        }
                    }
                }
            }

            if (!empty($this->m_data[APPID][$appid][QID][$qid][SUBQUERY]))
            {
                $recs = $this->m_data[APPID][$appid][QID][$qid][SUBQUERY];

                if (!empty($recs))
                {
                    foreach ($recs as $col_name => $col_expr)
                    {
                        $other_cols[$col_name] = $col_name;
                    }
                }
            }

            foreach ($tokens as $col_name)
            {
                if ($col_name = trim($col_name))
                {
                    if ($col_name == 'za')
                    {
                        $za = true;
                        continue;
                    }

                    $full_name = $table_name . '.' . $col_name;
                    $is_macro_col = strpos($col_name, $table_name . '_') === 0; // TODO: HACK: KLUGE: encapuslate
                    $col_var_name = $is_macro_col || !empty($other_cols[$col_name]) ? $col_name : $full_name ;

                    if ($za)
                    {
                        $col_var_name .= ' desc';
                        $za = false;
                    }

                    $alternate_order_expr .= $sep . $col_var_name;
                    $sep = ', ';
                }
            }

            if (!empty($alternate_order_expr))
            {
                return $alternate_order_expr;
            }
        }

        ////

        return !empty($this->m_data[APPID][$appid][QID][$qid][ORDER]) ? $this->m_data[APPID][$appid][QID][$qid][ORDER] : $table_name . '.' . $this->get_pk_name($table_name) ;
    }

    function get_macro($col, $alternate_qid = null)
    {
        $appid = $this->get_appid();
        $qid = $this->get_qid($alternate_qid);
        return !empty($this->m_data[APPID][$appid][QID][$qid][MACRO][$col]) ? $this->m_data[APPID][$appid][QID][$qid][MACRO][$col] : null ;
    }

    function get_list_macro($col, $alternate_qid = null)
    {
        $appid = $this->get_appid();
        $qid = $this->get_qid($alternate_qid);
        return !empty($this->m_data[APPID][$appid][QID][$qid][LIST_MACRO][$col]) ? $this->m_data[APPID][$appid][QID][$qid][LIST_MACRO][$col] : null;
    }

    function get_list_macro_order($col, $alternate_qid = null)
    {
        $appid = $this->get_appid();
        $qid = $this->get_qid($alternate_qid);
        return !empty($this->m_data[APPID][$appid][QID][$qid][LIST_MACRO_ORDER][$col]) ? $this->m_data[APPID][$appid][QID][$qid][LIST_MACRO_ORDER][$col] : null ;
    }

    function get_list_macro_filter($col, $alternate_qid = null)
    {
        $appid = $this->get_appid();
        $qid = $this->get_qid($alternate_qid);
        return !empty($this->m_data[APPID][$appid][QID][$qid][LIST_MACRO_FILTER][$col]) ? $this->m_data[APPID][$appid][QID][$qid][LIST_MACRO_FILTER][$col] : null ;
    }

    function get_extra($col, $alternate_qid = null)
    {
        $appid = $this->get_appid();
        $qid = $this->get_qid($alternate_qid);
        return !empty($this->m_data[APPID][$appid][QID][$qid][EXTRA][$col]) ? $this->m_data[APPID][$appid][QID][$qid][EXTRA][$col] : null ;
    }

    function get_detail_summary_cols($alternate_qid = null)
    {
        $appid = $this->get_appid();
        $qid = $this->get_qid($alternate_qid);

        $cols = null;

        if (!empty($this->m_data[APPID][$appid][QID][$qid][DETAIL_SUMMARY_COLS]))
        {
            foreach ($this->m_data[APPID][$appid][QID][$qid][DETAIL_SUMMARY_COLS] as $summary_col)
            {
                list($col_name, $group_col_name) = explode(',', $summary_col);

                if (($col_name = trim($col_name)))
                {
                    $cols[] = array($col_name, trim($group_col_name));
                }
            }
        }

        return $cols;
    }

    function is_form_omit_col($form_key, $col_name, $alternate_qid = null)
    {
        $appid = $this->get_appid();
        $qid = $this->get_qid($alternate_qid);

        $result = false;

        if (!empty($this->m_data[APPID][$appid][QID][$qid][OMIT][$form_key]))
        {
            $omit_cols = explode(',', $this->m_data[APPID][$appid][QID][$qid][OMIT][$form_key]);

            if (is_array($omit_cols) && !empty($omit_cols))
            {
                foreach ($omit_cols as $omit_col_name)
                {
                    if (trim($omit_col_name) == $col_name)
                    {
                        $result = true;
                        break;
                    }
                }
            }
        }

        return $result;
    }

    function is_nav_omit_tab($tab_name, $alternate_appid = null)
    {
        $appid = $this->get_appid($alternate_appid);

        $result = false;

        if (!empty($this->m_data[APPID][$appid][OMIT][NAV]))
        {
            $omit_tabs = explode(',', $this->m_data[APPID][$appid][OMIT][NAV]);

            if (is_array($omit_tabs) && !empty($omit_tabs))
            {
                foreach ($omit_tabs as $omit_tab_name)
                {
                    if (trim($omit_tab_name) == $tab_name)
                    {
                        $result = true;
                        break;
                    }
                }
            }
        }

        return $result;
    }

    function get_nav_incl_tabs($alternate_appid = null)
    {
        $appid = $this->get_appid($alternate_appid);

        $result = array();

        $incl_tabs = !empty($this->m_data[APPID][$appid][INCL][NAV]) ? explode(',', $this->m_data[APPID][$appid][INCL][NAV]) : null ;

        if (!empty($incl_tabs))
        {
            foreach ($incl_tabs as $incl_tab_name)
            {
                $incl_tab_name = trim($incl_tab_name);

                if (!empty($incl_tab_name))
                {
                    $result[] = $incl_tab_name;
                }
            }
        }

        return $result;
    }

    function is_rpt_omit_tab($tab_name, $alternate_appid = null)
    {
        $appid = $this->get_appid($alternate_appid);

        $result = false;

        if (!empty($this->m_data[APPID][$appid][OMIT][RPT]))
        {
            $omit_tabs = explode(',', $this->m_data[APPID][$appid][OMIT][RPT]);

            if (is_array($omit_tabs) && !empty($omit_tabs))
            {
                foreach ($omit_tabs as $omit_tab_name)
                {
                    if (trim($omit_tab_name) == $tab_name)
                    {
                        $result = true;
                        break;
                    }
                }
            }
        }

        return $result;
    }

    function get_rpt_incl_tabs($alternate_appid = null)
    {
        $appid = $this->get_appid($alternate_appid);

        $result = array();

        $incl_tabs = !empty($this->m_data[APPID][$appid][INCL][RPT]) ? explode(',', $this->m_data[APPID][$appid][INCL][RPT]) : null ;

        if (!empty($incl_tabs))
        {
            foreach ($incl_tabs as $incl_tab_name)
            {
                $incl_tab_name = trim($incl_tab_name);

                if (!empty($incl_tab_name))
                {
                    $result[] = $incl_tab_name;
                }
            }
        }

        return $result;
    }
}

////

?>
