<?php

/* report.php - SynApp2 PDF report generator
**
   +----------------------------------------------------------------------+
   | SynApp2 Version 1                                                    |
   +----------------------------------------------------------------------+
   | Copyright (c) 2007 - 2009 Richard Howell. All rights reserved.       |
   +----------------------------------------------------------------------+
   | This source file is subject to version 1.01 of the SynApp2 license,  |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
   | http://www.synapp2.org/license/1_01.txt                              |
   | If you did not receive a copy of the SynApp2 license and are unable  |
   | to obtain it through the world-wide-web, please send a note to       |
   | license@synapp2.org so we can mail you a copy immediately.           |
   +----------------------------------------------------------------------+
   | Authors: Richard Howell                                              |
   |                                                                      |
   +----------------------------------------------------------------------+
**
** http://www.synapp2.org
**
** $Id: report.php,v 1.4 2009/01/05 22:32:28 richard Exp $
*/

require('./3rdparty/fpdf.php');
require('./3rdparty/mc_table.php');

class report extends PDF_MC_Table // extends FPDF
{
    var $m_omit = array();
    var $m_hdgs = null;
    var $m_align = null;
    var $m_width = null;
    var $m_detail_summary = null;

    ////

    function report($orientation = 'l', $unit = 'pt', $format = 'letter')
    {
        $this->PDF_MC_Table($orientation, $unit, $format);
    }

    function em($mult = 1)
    {
        if (!is_numeric($mult) || $mult <= 0)
        {
            $mult = 1;
        }

        return $this->GetStringWidth('M') * ((float)$mult);
    }

    function auto_widths(&$data, $expand_to_fit = true)
    {
        $this->m_width = null;

        if (!empty($data))
        {
            $available = round(($this->w - $this->lMargin - $this->rMargin) / $this->em()); // line width
            $minsize = 0;
            $maxsize = 0;
            $colsize = array(); // starting size estimates
            $required = 0; // total of size estimates
            $deficit = 0;

            foreach ($data as $row)
            {
                $col_count = count($row) - count($this->m_omit);

                if (!$col_count)
                {
                    break; // TODO: TBD?? message??
                }

                $minsize = max(1, min(5, (int)($available / $col_count)));
                $maxsize = min(40, $minsize * 8);

                foreach ($row as $n => $v)
                {
                    if (!in_array($n, $this->m_omit))
                    {
                        $size = max($minsize, min($maxsize, strlen($v)));

                        if (!isset($colsize[$n]))
                        {
                            $colsize[$n] = $size;
                        }
                        else
                        {
                            $colsize[$n] = max($size, $colsize[$n]);
                        }
                    }
                }
            }

            $required = array_sum($colsize);

            if ($required)
            {
                $deficit = $required - $available;

                foreach ($colsize as $n => $size)
                {
                    $delta = ($size / $required) * $deficit;
                    $this->m_width[$n] = $this->em($size - ($expand_to_fit ? $delta : max(0, $delta)));
                }
            }
        }
    }

    function prepare_cols(&$data)
    {
        global $g_custom, $g_schema, $g_query;

        $this->m_omit = array();
        $this->m_hdgs = null;
        $this->m_align = null;
        $this->m_width = null;

        if (!empty($data))
        {
            $rec = $data[0];

            $map = $g_schema->get_col_display_name_map($g_query->get_table());

            foreach ($rec as $col_name => $col_value)
            {
                if ($g_schema->is_pk($g_query->get_table(), $col_name) ||
                    $g_schema->is_fk($g_query->get_table(), $col_name))
                {
                    $this->m_omit[] = $col_name;
                }
                else
                {
                    $this->m_hdgs[$col_name] = isset($map[$col_name]) ? $map[$col_name] : ucwords(str_replace('_', ' ', $col_name)) ;
                    $this->m_align[$col_name] = is_numeric($col_value) ? 'R' : 'L' ;
                }
            }

            $this->auto_widths($data);

            $this->m_detail_summary = new detail_summary($g_custom, $data);
        }
    }

    function get_col_values($rec)
    {
        $col_values = array();

        foreach ($rec as $n => $v)
        {
            if (!in_array($n, $this->m_omit))
            {
                $col_values[] = $v;
            }
        }

        return $col_values;
    }

    function Header()
    {
        if ($this->m_hdgs && is_array($this->m_hdgs))
        {
            $this->Row(array_values($this->m_hdgs));
        }
    }

    function Body(&$data)
    {
        if (!empty($data))
        {
            $this->prepare_cols($data);

            $this->SetAligns(array_values($this->m_align));
            $this->SetWidths(array_values($this->m_width));

            $this->AddPage();

            foreach ($data as $rec)
            {
                if ($summary_row = $this->m_detail_summary->get_summary_row($rec))
                {
                    $this->Row($this->get_col_values($summary_row));
                }

                $this->Row($this->get_col_values($rec));

                if ($summary_row = $this->m_detail_summary->get_final_row($rec))
                {
                    $this->Row($this->get_col_values($summary_row));
                }

                if ($summary_row = $this->m_detail_summary->get_grand_row($rec))
                {
                    $this->Row($this->get_col_values($summary_row));
                }
            }
        }
    }

    function Footer()
    {
        if (empty($this->AliasNbPages))
        {
            $this->AliasNbPages();
        }

        $this->SetY($this->bMargin / -2); // up from the bottom
        $this->Cell(0, 0, 'Page ' . $this->PageNo() . ' of ' . $this->AliasNbPages, 0, 0, 'C');
    }
};

////////

class detail_summary
{
    var $m_custom = null;

    var $m_rows_remaining = 0;
    var $m_col_names = array();
    var $m_detail_summary_cols = array();

    var $m_group_stat = array();
    var $m_grand_stat = array();

    function detail_summary(&$custom, &$data)
    {
        $this->m_custom = &$custom;
        $this->build($data);
    }

    function build(&$data)
    {
        if (!empty($data))
        {
            $this->m_rows_remaining = count($data);
            $this->m_col_names = array_keys($data[0]);
            $this->m_detail_summary_cols = $this->m_custom->get_detail_summary_cols();
        }
    }

    function reset_summary_row(&$summary_row)
    {
        if (!empty($this->m_col_names))
        {
            foreach ($this->m_col_names as $col_name)
            {
                $summary_row[$col_name] = '';
            }
        }
    }

    function update_col_stat(&$col_stat, &$rec, $col_name, $group_col_name = null)
    {
        $col_stat['count'] += 1;

        $col_value = 0 + $rec[$col_name];

        $col_stat['sum'] += $col_value;
        $col_stat['ave'] = $col_stat['sum'] / $col_stat['count'];

        if ($col_value < $col_stat['min'] || $col_stat['min'] === null)
        {
            $col_stat['min'] = $col_value;
        }

        if ($col_value > $col_stat['max'] || $col_stat['max'] === null)
        {
            $col_stat['max'] = $col_value;
        }

        if ($col_stat['last'] === null && isset($rec[$group_col_name]))
        {
            $col_stat['last'] = $rec[$group_col_name];
        }
    }

    function make_col_stat()
    {
        return array('count'=>0, 'sum'=>0, 'ave'=>0, 'min'=>null, 'max'=>null, 'last'=>null);
    }

    function tally_col_stat(&$rec, $col_name, $group_col_name = null)
    {
        $stat_rec = null;

        if ($col_name && isset($rec[$col_name]) && is_numeric($rec[$col_name])) // TBD: is_numeric() test not-needed or wrong place??
        {
            if (!isset($this->m_grand_stat[$col_name]))
            {
                $this->m_grand_stat[$col_name] = $this->make_col_stat();
            }

            $this->update_col_stat($this->m_grand_stat[$col_name], $rec, $col_name);

            ////////

            if ($group_col_name && isset($rec[$group_col_name]))
            {
                if (!isset($this->m_group_stat[$col_name][$group_col_name]))
                {
                    $this->m_group_stat[$col_name][$group_col_name] = $this->make_col_stat();
                }

                $col_stat = &$this->m_group_stat[$col_name][$group_col_name];

                if ($col_stat['last'] && $col_stat['last'] != $rec[$group_col_name])
                {
                    $stat_rec = $this->m_group_stat[$col_name][$group_col_name]; // copy by value

                    $this->m_group_stat[$col_name][$group_col_name] = $this->make_col_stat();

                    $col_stat = &$this->m_group_stat[$col_name][$group_col_name];
                }

                $this->update_col_stat($col_stat, $rec, $col_name, $group_col_name);
            }
        }

        return $stat_rec;
    }

    function get_group_stat($col_name, $group_col_name)
    {
        return isset($this->m_group_stat[$col_name][$group_col_name]) ? ($this->m_group_stat[$col_name][$group_col_name]) : null ;
    }

    function get_grand_stat($col_name)
    {
        return isset($this->m_grand_stat[$col_name]) ? ($this->m_grand_stat[$col_name]) : null ;
    }

    function get_summary_row(&$rec, $selector = null)
    {
        $summary_row = null;

        if (!empty($rec) && !empty($this->m_detail_summary_cols))
        {
            foreach ($this->m_detail_summary_cols as $summary_col)
            {
                $col_name = $summary_col[0];
                $group_col_name = $summary_col[1];

                $col_stat = null;

                if (!$selector)
                {
                    $col_stat = $this->tally_col_stat($rec, $col_name, $group_col_name);
                }
                else if (!$this->m_rows_remaining)
                {
                    if ($selector == 'final')
                    {
                        $col_stat = $this->get_group_stat($col_name, $group_col_name);
                    }
                    else if ($selector == 'grand')
                    {
                        $col_stat = $this->get_grand_stat($col_name);
                    }
                }

                if ($col_stat)
                {
                    if (!$summary_row)
                    {
                        $this->reset_summary_row($summary_row);
                    }

                    $summary_row[$col_name] = $col_stat['sum'];
                }
            }

            if ($this->m_rows_remaining)
            {
                $this->m_rows_remaining--;
            }
        }

        return $summary_row;
    }

    function get_final_row(&$rec)
    {
        return !$this->m_rows_remaining ? $this->get_summary_row($rec, 'final') : null ;
    }

    function get_grand_row(&$rec)
    {
        return !$this->m_rows_remaining ? $this->get_summary_row($rec, 'grand') : null ;
    }
};

////

?>
